/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.filestore;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.nrg.xnat.tags.TagLabelException;


/**
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public class MetadataDeleteOp implements MetadataOp {
    private static final String SQL_DELETE_METADATA_ALL = "DELETE from resource_tags WHERE resource_id = ? AND tag_label_id = ?";
    private static final String SQL_DELETE_METADATA = "DELETE from resource_tags WHERE resource_id = ? AND tag_label_id = ? AND value = ?";

    private final Connection c;
    private final int tagID;
    private final Collection<String> values;


    public MetadataDeleteOp(final Connection c, final int tagID, final String...values) {
	this.c = c;
	this.tagID = tagID;
	this.values = new ArrayList<String>(Arrays.asList(values));
    }


    public MetadataDeleteOp(final Connection c, final String label, final String...values)
    throws SQLException,TagLabelException {
	this(c, MetadataOp.Util.getLabelID(c, label));
    }


    /* (non-Javadoc)
     * @see org.nrg.xnat.filestore.MetadataOp#apply(java.net.URI)
     */
    public void apply(final int resourceID) throws SQLException {
	if (values.isEmpty()) {
	    final PreparedStatement delete = c.prepareStatement(SQL_DELETE_METADATA_ALL);
	    try {
		delete.setInt(1, resourceID);
		delete.setInt(2, tagID);
		delete.executeUpdate();
	    } finally {
		try { delete.close(); } catch (SQLException ignore) {}
	    }
	} else {
	    final PreparedStatement delete = c.prepareStatement(SQL_DELETE_METADATA);
	    try {
		for (final String value : values) {
		    delete.setInt(1, resourceID);
		    delete.setInt(2, tagID);
		    delete.setString(3, value);
		    delete.executeUpdate();	// TODO: check return value
		}
	    } finally {
		try { delete.close(); } catch (SQLException ignore) {}
	    }
	}
    }
}
